"""
This script is used to analyse GPCC data by calculating temporal statistics
over user defined time slices
- Outputs are tiff files that show the spatial distribution of the statistics
- To be used as one of the predictor variable in the Random Forest analysis

GPCC full data monthly on 0.5° resolution was used, unit: mm/mon:
https://opendata.dwd.de/climate_environment/GPCC/html/fulldata-monthly_v2018_doi_download.html


This script
- slices the data according to the time periods
- calculates absolute differences through period 2 - period 1 of
  . mean over the 30yr period
  . standard deviation of the 30yr period
  . average inter-annual min-max of the 30 year period
- writes arrays to output tiff based on the raster mask


Jens Kiesel
kiesel@igb-berlin.de
February 2020
"""


# import core modules
import logging

# import installed modules
import xarray as xr
import numpy as np

# import own modules
import GDAL_helper_functions as ghf
from config import Config as cfg

class GPCC_Processing(object):

    def __init__(self):
        # logging
        self.logger = logging.getLogger(cfg.LOGGER_NAME)
        self.logger.info(f'\nPreparing precipitation data')

    def workflow(self):
        self.logger.info(f'- opening data and reading time periods')
        # open NetCDF file and extract data for periods
        ds_in = xr.open_dataset(cfg.PCP_FP)
        ds_p1 = ds_in.sel(time=slice(cfg.START_PERIOD_1, cfg.END_PERIOD_1))
        ds_p2 = ds_in.sel(time=slice(cfg.START_PERIOD_2, cfg.END_PERIOD_2))

        self.logger.info(f'- calculating indices mean, std, annual-min-max')
        # calculate change indicators from period 1 to period 2 (negative is smaller in p2)
        # along axis = 0 = time
        diff_mean_arr = np.subtract(np.mean(ds_p2, 0), np.mean(ds_p1, 0))
        diff_std_arr = np.subtract(np.std(ds_p2, 0), np.std(ds_p1, 0))
        diff_ann_min_max_arr = np.subtract(np.subtract(np.max(ds_p2, 0), np.min(ds_p2, 0)),
                                           np.subtract(np.max(ds_p1, 0), np.min(ds_p1, 0)))

        self.logger.info(f'- writing output to {cfg.RANDOM_FOREST_PREDICTORS}')
        # write output based on mask (numpy array has to be 720 * 360)
        ghf.write_numpy_to_tiff(diff_mean_arr[cfg.PCP_NC_VAR].to_masked_array(), cfg.GEO_T, cfg.NX, cfg.NY, cfg.SRS,
                                str(diff_mean_arr[cfg.PCP_NC_VAR].dtype), cfg.PCP_DIFF_MEAN_FILE, cfg.NO_DATA)
        ghf.write_numpy_to_tiff(diff_std_arr[cfg.PCP_NC_VAR].to_masked_array(), cfg.GEO_T, cfg.NX, cfg.NY, cfg.SRS,
                                str(diff_std_arr[cfg.PCP_NC_VAR].dtype), cfg.PCP_DIFF_STD_FILE, cfg.NO_DATA)
        ghf.write_numpy_to_tiff(diff_ann_min_max_arr[cfg.PCP_NC_VAR].to_masked_array(), cfg.GEO_T, cfg.NX, cfg.NY, cfg.SRS,
                                str(diff_ann_min_max_arr[cfg.PCP_NC_VAR].dtype), cfg.PCP_OUT_DIFF_ANN_MIN_MAX_FILE, cfg.NO_DATA)


if __name__ == '__main__':
    tmp_processing = GPCC_Processing()
    tmp_processing.workflow()